@@ -640,6 +640,7 @@ nautilus_burn_recorder_write_tracks (NautilusBurnRecorder          *recorder,
 	NautilusBurnMediaType type;
 	gboolean needs_blank;
 	gboolean is_locked;
+	int ret;
 
 	g_return_val_if_fail (tracks != NULL, NAUTILUS_BURN_RECORDER_RESULT_ERROR);
 	recorder->priv->tracks = tracks;
@@ -663,6 +664,9 @@ nautilus_burn_recorder_write_tracks (NautilusBurnRecorder          *recorder,
 							  &needs_blank);
 
 	if (type == NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
+		if (is_locked)
+			nautilus_burn_drive_unlock (drive);
+
 		recorder->priv->result = NAUTILUS_BURN_RECORDER_RESULT_CANCEL;
 		return recorder->priv->result;
 	}
@@ -672,17 +676,20 @@ nautilus_burn_recorder_write_tracks (NautilusBurnRecorder          *recorder,
 	}
 
 	if (cd_write_needs_growisofs (drive, type, tracks)) {
-		return nautilus_burn_recorder_write_growisofs (recorder,
-							       drive, tracks, speed,
-							       flags);
-	} else {
-		return nautilus_burn_recorder_write_cdrecord (recorder,
+		ret = nautilus_burn_recorder_write_growisofs (recorder,
 							      drive, tracks, speed,
 							      flags);
+	} else {
+		ret = nautilus_burn_recorder_write_cdrecord (recorder,
+							     drive, tracks, speed,
+							     flags);
 	}
+
 	if (is_locked) {
 		nautilus_burn_drive_unlock (drive);
 	}
+
+	return ret;
 }
 
 static int
@@ -1035,6 +1042,9 @@ nautilus_burn_recorder_blank_disc_cdrecord (NautilusBurnRecorder         *record
 	media_type = nautilus_burn_recorder_wait_for_insertion (recorder, drive, &can_blank);
 
 	if (media_type == NAUTILUS_BURN_MEDIA_TYPE_ERROR || can_blank == FALSE) {
+		if (is_locked)
+			nautilus_burn_drive_unlock (drive);
+
 		recorder->priv->result = NAUTILUS_BURN_RECORDER_RESULT_CANCEL;
 		return recorder->priv->result;
 	}
